@@ -77,28 +77,41 @@ const QDBusArgument & operator>>(const QDBusArgument &arg, Property &change)
 MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     : QMainWindow(parent)
 {
+    qDebug() << "MainWindow::MainWindow(): Started MaemoBarcodeWindow";
+
+    recentDialog = NULL;
+    pluginsLoaded = false;
+
     // first of all, disable the camera app:
-    qDebug() << "Killing the camera app";
+    qDebug() << "MainWindow::MainWindow(): Killing the camera app";
     QProcess process;
     process.execute("/usr/sbin/dsmetool -k /usr/bin/camera-ui");
 
+    qDebug() << "MainWindow::MainWindow(): Setting window title";
+    setWindowTitle("Maemo Barcode Reader");
+
     // set up dbus
+    qDebug() << "MainWindow::MainWindow(): Setting up dbus";
     qDBusRegisterMetaType< Property >();
     qDBusRegisterMetaType< QList<Property> >();
+    qDebug() << "MainWindow::MainWindow(): Setting up lensCoverPropertyModified";
     QDBusConnection::systemBus().connect(QString(),
             DBUS_SHUTTER_STATE,
             "org.freedesktop.Hal.Device", "PropertyModified",
             this, SLOT(lensCoverPropertyModified(int, QList<Property>)));
+    qDebug() << "MainWindow::MainWindow(): Setting up focusButtonPropertyModified";
     QDBusConnection::systemBus().connect(QString(),
             DBUS_FOCUS_BUTTON,
             "org.freedesktop.Hal.Device", "PropertyModified",
             this, SLOT(focusButtonPropertyModified(int, QList<Property>)));
+    qDebug() << "MainWindow::MainWindow(): Setting up cameraButtonPropertyModified";
     QDBusConnection::systemBus().connect(QString(),
             DBUS_RELEASE_BUTTON,
             "org.freedesktop.Hal.Device", "PropertyModified",
             this, SLOT(cameraButtonPropertyModified(int,QList<Property>)));
 
     // init the state of the lens cover
+    qDebug() << "MainWindow::MainWindow(): Initing state of lens cover";
     initLensCoverState();
 
     // make sure the events are not called to pause the pipeline unecessary in the beginning
@@ -108,7 +121,7 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     // makes sure we don't try to do anything with a pointer to nowhere (using null-checks)
     videoWidget = NULL;
 
-    qDebug() << "In MainWindow::MainWindow";
+    qDebug() << "MainWindow::MainWindow(): Creating settings";
     settings = new QSettings("mbarcode", "mbarcode"); // load settings
     if(settings->value("settings_version", 1).toInt() < SETTINGS_VERSION) {
         settings->clear();
@@ -121,39 +134,37 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     qDebug() << "MainWindow::MainWindow(): Initializing results window";
     resultsWindow = new ResultsWindow(this); // create a resultswindow
 
-    qDebug() << "MainWindow::MainWindow(): Creating recent dialog";
-    recentDialog = new RecentDialog(this); // create a recent dialog with the tillRoll
-
     // set up the videoWidget
     qDebug() << "MainWindow::MainWindow(): Initializing video widget";
     videoWidget = new VideoWidget(this, decoder);
+    qDebug() << "MainWindow::MainWindow(): Setting video widget attributes";
     videoWidget->setAttribute(Qt::WA_NativeWindow, true);
     videoWidget->setContinuousFocusTime((int)(settings->value("autofocus_delay",2.0).toFloat()*1000)); // pull from the settings
+    qDebug() << "MainWindow::MainWindow(): Finished setting video widget attributes";
     //    videoWidget->setMinimumSize(300, 200);
 
     // connect the signal from the barcode decoder
+    qDebug() << "MainWindow::MainWindow(): Connecting signals";
     connect(decoder, SIGNAL(imageAnalysed(QString, QString, QImage*)),
             this, SLOT(imageAnalysed(QString, QString, QImage*)));
     // connect signals to disable/enable the refocus button
     connect(videoWidget, SIGNAL(focusingValueChanged(bool)), this, SLOT(setFocusing(bool)));
-
     connect(videoWidget, SIGNAL(doSaveBuffer()), this, SLOT(saveBufferImage()));
 
-    qDebug() << "MainWindow::MainWindow(): Loading plugins";
-    // load all plugins
-    loadPlugins(); // TODO !!!change this to only load those specified in the settings class!!!
-
     qDebug() << "MainWindow::MainWindow(): Initializing buttons";
     setAttribute(Qt::WA_Maemo5StackedWindow);
     setAttribute(Qt::WA_Maemo5AutoOrientation, true); // currently not working since libqt4-maemo5 is broken. Will work in PR1.2
 
     // if the lens cover is open, start scanning
+    qDebug() << "MainWindow::MainWindow(): Checking lens cover state";
     if (isLensCoverOpen()) {
         videoWidget->start(true); // start and keep running
     } else {
-        videoWidget->stop(); // make the screen go black, but make it start whenever the lens is opened
+        // We don't need to do anything here;
+        // wait until the lens cover is opened.
     }
 
+    qDebug() << "MainWindow::MainWindow(): Setting central widget";
     this->setCentralWidget(videoWidget);
     qDebug() << "MainWindow::MainWindow(): Finished";
 }
@@ -173,14 +184,17 @@ MaemoBarcodeWindow::~MaemoBarcodeWindow() {
 }
 
 void MaemoBarcodeWindow::showRecentDialog() {
-    int result = recentDialog->exec();
+    int result = getRecentDialog()->exec();
     if(result == QDialog::Accepted) {
-        emit barcodeAnalysedSignal(recentDialog->selectedBarcodeType(), recentDialog->selectedBarcodeData());
-        showResultsWindow(recentDialog->selectedBarcodeData());
+        ensurePluginsLoaded();
+        emit barcodeAnalysedSignal(getRecentDialog()->selectedBarcodeType(),
+                                   getRecentDialog()->selectedBarcodeData());
+        showResultsWindow(getRecentDialog()->selectedBarcodeData());
     }
 }
 
 void MaemoBarcodeWindow::sendBarcodeAnalysedSignal(QString barcodeType, QString barcodeData) {
+    ensurePluginsLoaded();
     emit barcodeAnalysedSignal(barcodeType, barcodeData); // emit a signal for whoever's listening. Especially plugins :)
 }
 
@@ -251,6 +265,22 @@ void MaemoBarcodeWindow::changeEvent(QEvent *e) {
     }
 }
 
+RecentDialog* MaemoBarcodeWindow::getRecentDialog()
+{
+    if (!recentDialog)
+        recentDialog = new RecentDialog(this);
+    return recentDialog;
+}
+
+void MaemoBarcodeWindow::ensurePluginsLoaded()
+{
+    if (!pluginsLoaded)
+    {
+        loadPlugins();
+        pluginsLoaded = true;
+    }
+}
+
 void MaemoBarcodeWindow::loadPlugins() {
     // TODO: Create a system which lets the user enable/disable plugins. These should not be loaded.
 
@@ -324,22 +354,24 @@ void MaemoBarcodeWindow::dataAnalysed(QString datatype, QString barcodeData, QHa
     // let's just send this data back to the other plugins
     // later on we should store this in a database which other plugins may use as lookup later
 
+    ensurePluginsLoaded();
     emit dataAnalysedSignal(datatype, barcodeData, data);
 }
 
 void MaemoBarcodeWindow::imageAnalysed(QString barcodeType, QString barcodeData, QImage *im) {
     videoWidget->stop(false); // cancel scanning first
 
+    ensurePluginsLoaded();
     emit barcodeAnalysedSignal(barcodeType, barcodeData); // emit a signal for whoever's listening. Especially plugins :)
 
     // display the output (this is done before anything is added to the tblTillRoll - avoids flickering)
     showResultsWindow(barcodeData);
 
-    recentDialog->imageAnalysed(barcodeType, barcodeData, im);
-
+    getRecentDialog()->imageAnalysed(barcodeType, barcodeData, im);
 }
 
 void MaemoBarcodeWindow::showResultsWindow(QString barcodeData) {
+    ensurePluginsLoaded();
     resultsWindow->show();
     resultsWindow->processBarcode(barcodeData);
 }
